#define MOZ_SOURCE_STAMP 4ef9e419a8e1a53a04300981e79c1f5d1f5047e5
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/4ef9e419a8e1a53a04300981e79c1f5d1f5047e5
