/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIDOMWindow.idl"

interface mozIDOMWindow;
interface mozIDOMWindowProxy;

/**
 * The nsIDOMUIEvent interface is the datatype for all UI events in the
 * Document Object Model.
 *
 * For more information on this interface please see
 * http://www.w3.org/TR/DOM-Level-2-Events/
 */

%{C++
namespace mozilla {
namespace dom {
class Event;
}
}
%}

[ptr] native EventPtr(mozilla::dom::Event);

[builtinclass, uuid(85ae52eb-37fa-4fd9-a2e6-c7d0f2a521b3)]
interface nsIDOMUIEvent : nsISupports
{
  readonly attribute mozIDOMWindowProxy view;
  readonly attribute long               detail;
  void                      initUIEvent(in DOMString typeArg,
                                        in boolean canBubbleArg,
                                        in boolean cancelableArg,
                                        in mozIDOMWindow viewArg,
                                        in long detailArg);

  const long SCROLL_PAGE_UP = -32768;
  const long SCROLL_PAGE_DOWN = 32768;

  readonly attribute long               layerX;
  readonly attribute long               layerY;
  readonly attribute long               pageX;
  readonly attribute long               pageY;
  readonly attribute unsigned long      which;
  readonly attribute nsIDOMNode         rangeParent;
  readonly attribute long               rangeOffset;

           attribute boolean            cancelBubble;

  readonly attribute boolean            isChar;

  [notxpcom, nostdcall] EventPtr AsEvent();
};
